import commands2
import hal
from util import Counter


def test_run_command(scheduler: commands2.CommandScheduler):
    counter = Counter()
    cmd = commands2.RunCommand(counter.increment)

    scheduler.schedule(cmd)
    scheduler.run()
    assert counter.value == 1

    hal.shutdown()

    scheduler.run()
    assert counter.value == 1



import commands2
import commands2.button

from util import Counter


class MyButton(commands2.button.Button):
    def __init__(self):
        super().__init__(self.isPressed)
        self.pressed = False

    def isPressed(self) -> bool:
        return self.pressed

    def setPressed(self, value: bool):
        self.pressed = value


class MyCommand(commands2.CommandBase):
    executed = 0

    ended = 0
    canceled = 0

    def execute(self) -> None:
        self.executed += 1

    def end(self, interrupted: bool) -> None:
        self.ended += 1
        if interrupted:
            self.canceled += 1


def test_when_pressed(scheduler: commands2.CommandScheduler):
    cmd1 = MyCommand()
    button = MyButton()
    button.setPressed(False)

    button.whenPressed(cmd1)
    scheduler.run()

    assert not cmd1.executed
    assert not scheduler.isScheduled(cmd1)

    button.setPressed(True)
    scheduler.run()
    scheduler.run()

    assert cmd1.executed
    assert scheduler.isScheduled(cmd1)


def test_when_released(scheduler: commands2.CommandScheduler):
    cmd1 = MyCommand()
    button = MyButton()
    button.setPressed(True)

    button.whenReleased(cmd1)
    scheduler.run()

    assert not cmd1.executed
    assert not scheduler.isScheduled(cmd1)

    button.setPressed(False)
    scheduler.run()
    scheduler.run()

    assert cmd1.executed
    assert scheduler.isScheduled(cmd1)


def test_while_held(scheduler: commands2.CommandScheduler):
    cmd1 = MyCommand()
    button = MyButton()
    button.setPressed(False)

    button.whileHeld(cmd1)
    scheduler.run()

    assert not cmd1.executed
    assert not scheduler.isScheduled(cmd1)

    button.setPressed(True)
    scheduler.run()
    scheduler.run()

    assert cmd1.executed == 2
    assert scheduler.isScheduled(cmd1)

    button.setPressed(False)
    scheduler.run()

    assert cmd1.executed == 2
    assert not scheduler.isScheduled(cmd1)


def test_when_held(scheduler: commands2.CommandScheduler):
    cmd1 = MyCommand()
    button = MyButton()
    button.setPressed(False)

    button.whenHeld(cmd1)
    scheduler.run()

    assert not cmd1.executed
    assert not scheduler.isScheduled(cmd1)

    button.setPressed(True)
    scheduler.run()
    scheduler.run()

    assert cmd1.executed == 2
    assert scheduler.isScheduled(cmd1)

    button.setPressed(False)
    scheduler.run()

    assert cmd1.executed == 2
    assert not scheduler.isScheduled(cmd1)


def test_toggle_when_pressed(scheduler: commands2.CommandScheduler):
    cmd1 = MyCommand()
    button = MyButton()
    button.setPressed(False)

    button.toggleWhenPressed(cmd1)
    scheduler.run()

    assert not cmd1.executed
    assert not scheduler.isScheduled(cmd1)

    button.setPressed(True)
    scheduler.run()

    assert cmd1.executed
    assert scheduler.isScheduled(cmd1)


def test_cancel_when_pressed(scheduler: commands2.CommandScheduler):
    cmd1 = MyCommand()
    button = MyButton()
    button.setPressed(False)

    scheduler.schedule(cmd1)

    button.cancelWhenPressed(cmd1)
    scheduler.run()

    assert cmd1.executed == 1
    assert cmd1.ended == 0
    assert cmd1.canceled == 0
    assert scheduler.isScheduled(cmd1)

    button.setPressed(True)
    scheduler.run()
    scheduler.run()

    assert cmd1.executed == 1
    assert cmd1.ended == 1
    assert cmd1.canceled == 1
    assert not scheduler.isScheduled(cmd1)


def test_function_bindings(scheduler: commands2.CommandScheduler):

    buttonWhenPressed = MyButton()
    buttonWhileHeld = MyButton()
    buttonWhenReleased = MyButton()

    buttonWhenPressed.setPressed(False)
    buttonWhileHeld.setPressed(True)
    buttonWhenReleased.setPressed(True)

    counter = Counter()

    buttonWhenPressed.whenPressed(counter.increment)
    buttonWhileHeld.whileHeld(counter.increment)
    buttonWhenReleased.whenReleased(counter.increment)

    scheduler.run()
    buttonWhenPressed.setPressed(True)
    buttonWhenReleased.setPressed(False)
    scheduler.run()

    assert counter.value == 4


def test_button_composition(scheduler: commands2.CommandScheduler):

    button1 = MyButton()
    button2 = MyButton()

    button1.setPressed(True)
    button2.setPressed(False)

    # TODO: not sure if this is a great idea?
    assert button1
    assert not button2

    assert not button1.and_(button2)
    assert button1.or_(button2)
    assert not button1.not_()
    assert button1.and_(button2.not_())



import pytest

import commands2
from wpilib.simulation import DriverStationSim


@pytest.fixture(autouse=True)
def scheduler():
    commands2.CommandScheduler.resetInstance()
    DriverStationSim.setEnabled(True)
    DriverStationSim.notifyNewData()
    return commands2.CommandScheduler.getInstance()



# autogenerated by 'robotpy-build create-imports commands2.cmd commands2._impl.cmd'
from .._impl.cmd import (
    deadline,
    either,
    nothing,
    parallel,
    print,
    race,
    repeatingSequence,
    run,
    runEnd,
    runOnce,
    select,
    sequence,
    startEnd,
    wait,
    waitUntil,
)

__all__ = [
    "deadline",
    "either",
    "nothing",
    "parallel",
    "print",
    "race",
    "repeatingSequence",
    "run",
    "runEnd",
    "runOnce",
    "select",
    "sequence",
    "startEnd",
    "wait",
    "waitUntil",
]



from . import _init_impl

from .version import version as __version__

# autogenerated by 'robotpy-build create-imports commands2 commands2._impl'
from ._impl import (
    Command,
    CommandBase,
    CommandGroupBase,
    CommandScheduler,
    ConditionalCommand,
    FunctionalCommand,
    InstantCommand,
    MecanumControllerCommand,
    NotifierCommand,
    PIDCommand,
    PIDSubsystem,
    ParallelCommandGroup,
    ParallelDeadlineGroup,
    ParallelRaceGroup,
    PerpetualCommand,
    PrintCommand,
    ProfiledPIDCommand,
    ProfiledPIDSubsystem,
    ProxyScheduleCommand,
    RamseteCommand,
    RepeatCommand,
    RunCommand,
    ScheduleCommand,
    SelectCommand,
    SequentialCommandGroup,
    StartEndCommand,
    Subsystem,
    SubsystemBase,
    Swerve2ControllerCommand,
    Swerve3ControllerCommand,
    Swerve4ControllerCommand,
    Swerve6ControllerCommand,
    TimedCommandRobot,
    TrapezoidProfileCommand,
    TrapezoidProfileCommandRadians,
    TrapezoidProfileSubsystem,
    TrapezoidProfileSubsystemRadians,
    Trigger,
    WaitCommand,
    WaitUntilCommand,
    # button,
    # cmd,
    requirementsDisjoint,
)

__all__ = [
    "Command",
    "CommandBase",
    "CommandGroupBase",
    "CommandScheduler",
    "ConditionalCommand",
    "SelectCommand",
    "FunctionalCommand",
    "InstantCommand",
    "MecanumControllerCommand",
    "NotifierCommand",
    "PIDCommand",
    "PIDSubsystem",
    "ParallelCommandGroup",
    "ParallelDeadlineGroup",
    "ParallelRaceGroup",
    "PerpetualCommand",
    "PrintCommand",
    "ProfiledPIDCommand",
    "ProfiledPIDSubsystem",
    "ProxyScheduleCommand",
    "RamseteCommand",
    "RepeatCommand",
    "RunCommand",
    "ScheduleCommand",
    "SequentialCommandGroup",
    "StartEndCommand",
    "Subsystem",
    "SubsystemBase",
    "Swerve2ControllerCommand",
    "Swerve3ControllerCommand",
    "Swerve4ControllerCommand",
    "Swerve6ControllerCommand",
    "TimedCommandRobot",
    "TrapezoidProfileCommand",
    "TrapezoidProfileCommandRadians",
    "TrapezoidProfileSubsystem",
    "TrapezoidProfileSubsystemRadians",
    "Trigger",
    "WaitCommand",
    "WaitUntilCommand",
    # "button",
    # "cmd",
    "requirementsDisjoint",
]



import commands2
from util import ConditionHolder


def test_instant_command(scheduler: commands2.CommandScheduler):
    cond = ConditionHolder()

    cmd = commands2.InstantCommand(cond.setTrue)

    scheduler.schedule(cmd)
    scheduler.run()

    assert cond.getCondition()
    assert not scheduler.isScheduled(cmd)



import commands2
from commands2 import WaitCommand

import pytest


class MySubsystem(commands2.SubsystemBase):
    def __init__(self, param) -> None:
        super().__init__()
        self.param = param


class MyCommand(commands2.CommandBase):
    def __init__(self, param) -> None:
        super().__init__()
        self.addRequirements(MySubsystem(param))


def get_params(g):
    return sorted(s.param for s in g.getRequirements())


@pytest.mark.parametrize(
    "ccls",
    [
        commands2.ParallelCommandGroup,
        lambda *args: commands2.ParallelDeadlineGroup(WaitCommand(1), *args),
        commands2.ParallelRaceGroup,
        commands2.SequentialCommandGroup,
    ],
)
def test_command_group_constructors(ccls):
    # individual param form
    g = ccls(MyCommand(1))
    assert get_params(g) == [1]

    g = ccls(MyCommand(1), MyCommand(2))
    assert get_params(g) == [1, 2]

    # list form
    g = ccls([MyCommand(1)])
    assert get_params(g) == [1]

    g = ccls([MyCommand(1), MyCommand(2)])
    assert get_params(g) == [1, 2]

    # tuple form
    g = ccls((MyCommand(1),))
    assert get_params(g) == [1]

    g = ccls((MyCommand(1), MyCommand(2)))
    assert get_params(g) == [1, 2]


@pytest.mark.parametrize(
    "ccls",
    [
        commands2.ParallelCommandGroup,
        lambda *args: commands2.ParallelDeadlineGroup(WaitCommand(1), *args),
        commands2.ParallelRaceGroup,
        commands2.SequentialCommandGroup,
    ],
)
def test_command_group_add_commands(ccls):
    g = ccls()

    # individual param form
    g.addCommands(WaitCommand(1))
    g.addCommands(WaitCommand(1), WaitCommand(2))

    # list form
    g.addCommands([WaitCommand(1)])
    g.addCommands([WaitCommand(1), WaitCommand(2)])

    # tuple form
